@@ -1,13 +1,17 @@
-// Copyright (C) 2008-2016 National ICT Australia (NICTA)
+// Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
+// Copyright 2008-2016 National ICT Australia (NICTA)
 // 
-// This Source Code Form is subject to the terms of the Mozilla Public
-// License, v. 2.0. If a copy of the MPL was not distributed with this
-// file, You can obtain one at http://mozilla.org/MPL/2.0/.
-// -------------------------------------------------------------------
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+// http://www.apache.org/licenses/LICENSE-2.0
 // 
-// Written by Conrad Sanderson - http://conradsanderson.id.au
-// Written by Ryan Curtin
-// Written by Matthew Amidon
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+// ------------------------------------------------------------------------
 
 
 //! \addtogroup SpMat
@@ -32,47 +36,47 @@ class SpMat : public SpBase< eT, SpMat<eT> >
   const uword n_nonzero; //!< number of nonzero elements (read-only)
   const uword vec_state; //!< 0: matrix; 1: column vector; 2: row vector
   
-  // so that SpValProxy can call add_element() and delete_element()
-  friend class SpValProxy<SpMat<eT> >;
-  friend class SpSubview<eT>;
   
-  /**
-   * The memory used to store the values of the matrix.
-   * In accordance with the CSC format, this stores only the actual values.
-   * The correct locations of the values are assembled from the row indices
-   * and the column pointers.
-   * 
-   * The length of this array is (n_nonzero + 1); the final value ensures
-   * the integrity of iterators.  If you are planning on resizing this vector,
-   * it's probably best to use mem_resize() instead, which automatically sets
-   * the length to (n_nonzero + 1).  If you need to allocate the memory yourself
-   * for some reason, be sure to set values[n_nonzero] to 0.
-   */
+  // The memory used to store the values of the matrix.
+  // In accordance with the CSC format, this stores only the actual values.
+  // The correct locations of the values are assembled from the row indices and column pointers.
+  // 
+  // The length of this array is (n_nonzero + 1).
+  // The final value values[n_nonzero] must be zero to ensure integrity of iterators.
+  // Use mem_resize(new_n_nonzero) to resize this array.
+  // 
+  // WARNING: the 'values' array is only valid after sync() is called;
+  // WARNING: there is a separate cache for fast element insertion
+  
   arma_aligned const eT* const values;
   
-  /**
-   * The row indices of each value.  row_indices[i] is the row of values[i].
-   * 
-   * The length of this array is (n_nonzero + 1); the final value ensures
-   * the integrity of iterators.  If you are planning on resizing this vector,
-   * it's probably best to use mem_resize() instead.  If you need to allocate
-   * the memory yourself for some reason, be sure to set row_indices[n_nonzero] to 0.
-   */
+  
+  // The row indices of each value.  row_indices[i] is the row of values[i].
+  // 
+  // The length of this array is (n_nonzero + 1).
+  // The final value row_indices[n_nonzero] must be zero to ensure integrity of iterators.
+  // Use mem_resize(new_n_nonzero) to resize this array.
+  // 
+  // WARNING: the 'row_indices' array is only valid after sync() is called;
+  // WARNING: there is a separate cache for fast element insertion
+  
   arma_aligned const uword* const row_indices;
   
-  /**
-   * The column pointers.  This stores the index of the first item in column i.
-   * That is, values[col_ptrs[i]] is the first value in column i, and it is in
-   * the row indicated by row_indices[col_ptrs[i]].
-   * 
-   * This array is of length (n_cols + 2); the element col_ptrs[n_cols] should
-   * be equal to n_nonzero, and the element col_ptrs[n_cols + 1] is an invalid
-   * very large value that ensures the integrity of iterators.
-   * 
-   * The col_ptrs array is set by the init() function (which is called by the
-   * constructors and set_size() and other functions that set the size of the
-   * matrix), so allocating col_ptrs by hand should not be necessary.
-   */
+  
+  // The column pointers.  This stores the index of the first item in column i.
+  // That is, values[col_ptrs[i]] is the first value in column i,
+  // and it is in the row indicated by row_indices[col_ptrs[i]].
+  // 
+  // The length of this array is (n_cols + 2).
+  // The element col_ptrs[n_cols] must be equal to n_nonzero.
+  // The element col_ptrs[n_cols + 1] must be an invalid very large value to ensure integrity of iterators.
+  // 
+  // The col_ptrs array is set by the init() function
+  // (which is called by constructors, set_size() and other functions that change the matrix size).
+  // 
+  // WARNING: the 'col_ptrs' array is only valid after sync() is called;
+  // WARNING: there is a separate cache for fast element insertion
+  
   arma_aligned const uword* const col_ptrs;
   
   inline  SpMat();
@@ -81,18 +85,20 @@ class SpMat : public SpBase< eT, SpMat<eT> >
   inline explicit SpMat(const uword in_rows, const uword in_cols);
   inline explicit SpMat(const SizeMat& s);
   
-  inline                  SpMat(const char*        text);
-  inline const SpMat& operator=(const char*        text);
-  inline                  SpMat(const std::string& text);
-  inline const SpMat& operator=(const std::string& text);
-  inline                  SpMat(const SpMat<eT>&   x);
-  
+  inline            SpMat(const char*        text);
+  inline SpMat& operator=(const char*        text);
+  inline            SpMat(const std::string& text);
+  inline SpMat& operator=(const std::string& text);
+  inline            SpMat(const SpMat<eT>&   x);
   
   #if defined(ARMA_USE_CXX11)
-  inline                  SpMat(SpMat&& m);
-  inline const SpMat& operator=(SpMat&& m);
+  inline            SpMat(SpMat&& m);
+  inline SpMat& operator=(SpMat&& m);
   #endif
   
+  inline explicit    SpMat(const MapMat<eT>& x);
+  inline SpMat&  operator=(const MapMat<eT>& x);
+  
   template<typename T1, typename T2, typename T3>
   inline SpMat(const Base<uword,T1>& rowind, const Base<uword,T2>& colptr, const Base<eT,T3>& values, const uword n_rows, const uword n_cols);
   
@@ -105,65 +111,73 @@ class SpMat : public SpBase< eT, SpMat<eT> >
   template<typename T1, typename T2>
   inline SpMat(const bool add_values, const Base<uword,T1>& locations, const Base<eT,T2>& values, const uword n_rows, const uword n_cols, const bool sort_locations = true, const bool check_for_zeros = true);
   
-  inline const SpMat&  operator=(const eT val); //! sets size to 1x1
-  inline const SpMat& operator*=(const eT val);
-  inline const SpMat& operator/=(const eT val);
+  inline SpMat&  operator=(const eT val); //! sets size to 1x1
+  inline SpMat& operator*=(const eT val);
+  inline SpMat& operator/=(const eT val);
   // operator+=(val) and operator-=(val) are not defined as they don't make sense for sparse matrices
   
-  inline const SpMat&  operator=(const SpMat& m);
-  inline const SpMat& operator+=(const SpMat& m);
-  inline const SpMat& operator-=(const SpMat& m);
-  inline const SpMat& operator*=(const SpMat& m);
-  inline const SpMat& operator%=(const SpMat& m);
-  inline const SpMat& operator/=(const SpMat& m);
+  inline SpMat&  operator=(const SpMat& m);
+  inline SpMat& operator+=(const SpMat& m);
+  inline SpMat& operator-=(const SpMat& m);
+  inline SpMat& operator*=(const SpMat& m);
+  inline SpMat& operator%=(const SpMat& m);
+  inline SpMat& operator/=(const SpMat& m);
   
-  template<typename T1> inline explicit          SpMat(const Base<eT, T1>& m);
-  template<typename T1> inline const SpMat&  operator=(const Base<eT, T1>& m);
-  template<typename T1> inline const SpMat& operator+=(const Base<eT, T1>& m);
-  template<typename T1> inline const SpMat& operator-=(const Base<eT, T1>& m);
-  template<typename T1> inline const SpMat& operator*=(const Base<eT, T1>& m);
-  template<typename T1> inline const SpMat& operator/=(const Base<eT, T1>& m);
-  template<typename T1> inline const SpMat& operator%=(const Base<eT, T1>& m);
+  template<typename T1> inline explicit    SpMat(const Base<eT, T1>& m);
+  template<typename T1> inline SpMat&  operator=(const Base<eT, T1>& m);
+  template<typename T1> inline SpMat& operator+=(const Base<eT, T1>& m);
+  template<typename T1> inline SpMat& operator-=(const Base<eT, T1>& m);
+  template<typename T1> inline SpMat& operator*=(const Base<eT, T1>& m);
+  template<typename T1> inline SpMat& operator/=(const Base<eT, T1>& m);
+  template<typename T1> inline SpMat& operator%=(const Base<eT, T1>& m);
   
   
   //! construction of complex matrix out of two non-complex matrices
   template<typename T1, typename T2>
   inline explicit SpMat(const SpBase<pod_type, T1>& A, const SpBase<pod_type, T2>& B);
   
-  inline                   SpMat(const SpSubview<eT>& X);
-  inline const SpMat&  operator=(const SpSubview<eT>& X);
-  inline const SpMat& operator+=(const SpSubview<eT>& X);
-  inline const SpMat& operator-=(const SpSubview<eT>& X);
-  inline const SpMat& operator*=(const SpSubview<eT>& X);
-  inline const SpMat& operator%=(const SpSubview<eT>& X);
-  inline const SpMat& operator/=(const SpSubview<eT>& X);
+  inline             SpMat(const SpSubview<eT>& X);
+  inline SpMat&  operator=(const SpSubview<eT>& X);
+  inline SpMat& operator+=(const SpSubview<eT>& X);
+  inline SpMat& operator-=(const SpSubview<eT>& X);
+  inline SpMat& operator*=(const SpSubview<eT>& X);
+  inline SpMat& operator%=(const SpSubview<eT>& X);
+  inline SpMat& operator/=(const SpSubview<eT>& X);
+  
+  inline             SpMat(const spdiagview<eT>& X);
+  inline SpMat&  operator=(const spdiagview<eT>& X);
+  inline SpMat& operator+=(const spdiagview<eT>& X);
+  inline SpMat& operator-=(const spdiagview<eT>& X);
+  inline SpMat& operator*=(const spdiagview<eT>& X);
+  inline SpMat& operator%=(const spdiagview<eT>& X);
+  inline SpMat& operator/=(const spdiagview<eT>& X);
   
   // delayed unary ops
-  template<typename T1, typename spop_type> inline                   SpMat(const SpOp<T1, spop_type>& X);
-  template<typename T1, typename spop_type> inline const SpMat&  operator=(const SpOp<T1, spop_type>& X);
-  template<typename T1, typename spop_type> inline const SpMat& operator+=(const SpOp<T1, spop_type>& X);
-  template<typename T1, typename spop_type> inline const SpMat& operator-=(const SpOp<T1, spop_type>& X);
-  template<typename T1, typename spop_type> inline const SpMat& operator*=(const SpOp<T1, spop_type>& X);
-  template<typename T1, typename spop_type> inline const SpMat& operator%=(const SpOp<T1, spop_type>& X);
-  template<typename T1, typename spop_type> inline const SpMat& operator/=(const SpOp<T1, spop_type>& X);
+  template<typename T1, typename spop_type> inline             SpMat(const SpOp<T1, spop_type>& X);
+  template<typename T1, typename spop_type> inline SpMat&  operator=(const SpOp<T1, spop_type>& X);
+  template<typename T1, typename spop_type> inline SpMat& operator+=(const SpOp<T1, spop_type>& X);
+  template<typename T1, typename spop_type> inline SpMat& operator-=(const SpOp<T1, spop_type>& X);
+  template<typename T1, typename spop_type> inline SpMat& operator*=(const SpOp<T1, spop_type>& X);
+  template<typename T1, typename spop_type> inline SpMat& operator%=(const SpOp<T1, spop_type>& X);
+  template<typename T1, typename spop_type> inline SpMat& operator/=(const SpOp<T1, spop_type>& X);
   
   // delayed binary ops
-  template<typename T1, typename T2, typename spglue_type> inline                   SpMat(const SpGlue<T1, T2, spglue_type>& X);
-  template<typename T1, typename T2, typename spglue_type> inline const SpMat&  operator=(const SpGlue<T1, T2, spglue_type>& X);
-  template<typename T1, typename T2, typename spglue_type> inline const SpMat& operator+=(const SpGlue<T1, T2, spglue_type>& X);
-  template<typename T1, typename T2, typename spglue_type> inline const SpMat& operator-=(const SpGlue<T1, T2, spglue_type>& X);
-  template<typename T1, typename T2, typename spglue_type> inline const SpMat& operator*=(const SpGlue<T1, T2, spglue_type>& X);
-  template<typename T1, typename T2, typename spglue_type> inline const SpMat& operator%=(const SpGlue<T1, T2, spglue_type>& X);
-  template<typename T1, typename T2, typename spglue_type> inline const SpMat& operator/=(const SpGlue<T1, T2, spglue_type>& X);
+  template<typename T1, typename T2, typename spglue_type> inline             SpMat(const SpGlue<T1, T2, spglue_type>& X);
+  template<typename T1, typename T2, typename spglue_type> inline SpMat&  operator=(const SpGlue<T1, T2, spglue_type>& X);
+  template<typename T1, typename T2, typename spglue_type> inline SpMat& operator+=(const SpGlue<T1, T2, spglue_type>& X);
+  template<typename T1, typename T2, typename spglue_type> inline SpMat& operator-=(const SpGlue<T1, T2, spglue_type>& X);
+  template<typename T1, typename T2, typename spglue_type> inline SpMat& operator*=(const SpGlue<T1, T2, spglue_type>& X);
+  template<typename T1, typename T2, typename spglue_type> inline SpMat& operator%=(const SpGlue<T1, T2, spglue_type>& X);
+  template<typename T1, typename T2, typename spglue_type> inline SpMat& operator/=(const SpGlue<T1, T2, spglue_type>& X);
   
   // delayed mixed-type unary ops
-  template<typename T1, typename spop_type> inline                   SpMat(const mtSpOp<eT, T1, spop_type>& X);
-  template<typename T1, typename spop_type> inline const SpMat&  operator=(const mtSpOp<eT, T1, spop_type>& X);
-  template<typename T1, typename spop_type> inline const SpMat& operator+=(const mtSpOp<eT, T1, spop_type>& X);
-  template<typename T1, typename spop_type> inline const SpMat& operator-=(const mtSpOp<eT, T1, spop_type>& X);
-  template<typename T1, typename spop_type> inline const SpMat& operator*=(const mtSpOp<eT, T1, spop_type>& X);
-  template<typename T1, typename spop_type> inline const SpMat& operator%=(const mtSpOp<eT, T1, spop_type>& X);
-  template<typename T1, typename spop_type> inline const SpMat& operator/=(const mtSpOp<eT, T1, spop_type>& X);
+  template<typename T1, typename spop_type> inline             SpMat(const mtSpOp<eT, T1, spop_type>& X);
+  template<typename T1, typename spop_type> inline SpMat&  operator=(const mtSpOp<eT, T1, spop_type>& X);
+  template<typename T1, typename spop_type> inline SpMat& operator+=(const mtSpOp<eT, T1, spop_type>& X);
+  template<typename T1, typename spop_type> inline SpMat& operator-=(const mtSpOp<eT, T1, spop_type>& X);
+  template<typename T1, typename spop_type> inline SpMat& operator*=(const mtSpOp<eT, T1, spop_type>& X);
+  template<typename T1, typename spop_type> inline SpMat& operator%=(const mtSpOp<eT, T1, spop_type>& X);
+  template<typename T1, typename spop_type> inline SpMat& operator/=(const mtSpOp<eT, T1, spop_type>& X);
   
   
   arma_inline       SpSubview<eT> row(const uword row_num);
@@ -228,18 +242,18 @@ class SpMat : public SpBase< eT, SpMat<eT> >
   
   
   // access the i-th element; if there is nothing at element i, 0 is returned
-  arma_inline arma_warn_unused SpValProxy<SpMat<eT> > operator[] (const uword i);
-  arma_inline arma_warn_unused eT                     operator[] (const uword i) const;
-  arma_inline arma_warn_unused SpValProxy<SpMat<eT> > at         (const uword i);
-  arma_inline arma_warn_unused eT                     at         (const uword i) const;
-  arma_inline arma_warn_unused SpValProxy<SpMat<eT> > operator() (const uword i);
-  arma_inline arma_warn_unused eT                     operator() (const uword i) const;
+  arma_inline arma_warn_unused MapMat_elem<eT> operator[] (const uword i);
+  arma_inline arma_warn_unused eT              operator[] (const uword i) const;
+  arma_inline arma_warn_unused MapMat_elem<eT> at         (const uword i);
+  arma_inline arma_warn_unused eT              at         (const uword i) const;
+  arma_inline arma_warn_unused MapMat_elem<eT> operator() (const uword i);
+  arma_inline arma_warn_unused eT              operator() (const uword i) const;
   
   // access the element at the given row and column; if there is nothing at that position, 0 is returned
-  arma_inline arma_warn_unused SpValProxy<SpMat<eT> > at         (const uword in_row, const uword in_col);
-  arma_inline arma_warn_unused eT                     at         (const uword in_row, const uword in_col) const;
-  arma_inline arma_warn_unused SpValProxy<SpMat<eT> > operator() (const uword in_row, const uword in_col);
-  arma_inline arma_warn_unused eT                     operator() (const uword in_row, const uword in_col) const;
+  arma_inline arma_warn_unused MapMat_elem<eT> at         (const uword in_row, const uword in_col);
+  arma_inline arma_warn_unused eT              at         (const uword in_row, const uword in_col) const;
+  arma_inline arma_warn_unused MapMat_elem<eT> operator() (const uword in_row, const uword in_col);
+  arma_inline arma_warn_unused eT              operator() (const uword in_row, const uword in_col) const;
   
   
   arma_inline arma_warn_unused bool is_empty()  const;
@@ -319,22 +333,20 @@ class SpMat : public SpBase< eT, SpMat<eT> >
   
   
   // saving and loading
-  
-  inline bool save(const std::string   name, const file_type type = arma_binary, const bool print_status = true) const;
-  inline bool save(      std::ostream& os,   const file_type type = arma_binary, const bool print_status = true) const;
-  
-  inline bool load(const std::string   name, const file_type type = arma_binary, const bool print_status = true);
-  inline bool load(      std::istream& is,   const file_type type = arma_binary, const bool print_status = true);
-  
-  inline bool quiet_save(const std::string   name, const file_type type = arma_binary) const;
-  inline bool quiet_save(      std::ostream& os,   const file_type type = arma_binary) const;
-  
-  inline bool quiet_load(const std::string   name, const file_type type = arma_binary);
-  inline bool quiet_load(      std::istream& is,   const file_type type = arma_binary);
-  
-  // TODO: speed up loading of sparse matrices stored as text files (ie. raw_ascii and coord_ascii)
   // TODO: implement auto_detect for sparse matrices
-  // TODO: modify docs to specify which formats are not applicable to sparse matrices
+  
+  inline arma_cold bool save(const std::string   name, const file_type type = arma_binary, const bool print_status = true) const;
+  inline arma_cold bool save(      std::ostream& os,   const file_type type = arma_binary, const bool print_status = true) const;
+  
+  inline arma_cold bool load(const std::string   name, const file_type type = arma_binary, const bool print_status = true);
+  inline arma_cold bool load(      std::istream& is,   const file_type type = arma_binary, const bool print_status = true);
+  
+  inline arma_cold bool quiet_save(const std::string   name, const file_type type = arma_binary) const;
+  inline arma_cold bool quiet_save(      std::ostream& os,   const file_type type = arma_binary) const;
+  
+  inline arma_cold bool quiet_load(const std::string   name, const file_type type = arma_binary);
+  inline arma_cold bool quiet_load(      std::istream& is,   const file_type type = arma_binary);
+  
   
   
   // necessary forward declarations
@@ -353,7 +365,7 @@ class SpMat : public SpBase< eT, SpMat<eT> >
     inline iterator_base(const SpMat& in_M);
     inline iterator_base(const SpMat& in_M, const uword col, const uword pos);
     
-    inline arma_hot eT operator*() const;
+    arma_inline eT operator*() const;
     
     // don't hold location internally; call "dummy" methods to get that information
     arma_inline uword row() const { return M->row_indices[internal_pos]; }
@@ -364,10 +376,9 @@ class SpMat : public SpBase< eT, SpMat<eT> >
     arma_aligned       uword  internal_col;
     arma_aligned       uword  internal_pos;
     
-    // so that we satisfy the STL iterator types
     typedef std::bidirectional_iterator_tag iterator_category;
     typedef eT                              value_type;
-    typedef uword                           difference_type; // not certain on this one
+    typedef std::ptrdiff_t                  difference_type;  // TODO: not certain on this one
     typedef const eT*                       pointer;
     typedef const eT&                       reference;
     };
@@ -384,11 +395,11 @@ class SpMat : public SpBase< eT, SpMat<eT> >
     inline const_iterator(const SpMat& in_M, uword in_row, uword in_col, uword in_pos);
     inline const_iterator(const const_iterator& other);
     
-    inline arma_hot const_iterator& operator++();
-    inline arma_hot const_iterator  operator++(int);
+    inline arma_hot         const_iterator& operator++();
+    inline arma_warn_unused const_iterator  operator++(int);
     
-    inline arma_hot const_iterator& operator--();
-    inline arma_hot const_iterator  operator--(int);
+    inline arma_hot         const_iterator& operator--();
+    inline arma_warn_unused const_iterator  operator--(int);
     
     inline arma_hot bool operator==(const const_iterator& rhs) const;
     inline arma_hot bool operator!=(const const_iterator& rhs) const;
@@ -421,11 +432,11 @@ class SpMat : public SpBase< eT, SpMat<eT> >
     inline arma_hot SpValProxy<SpMat<eT> > operator*();
     
     // overloads needed for return type correctness
-    inline arma_hot iterator& operator++();
-    inline arma_hot iterator  operator++(int);
+    inline arma_hot         iterator& operator++();
+    inline arma_warn_unused iterator  operator++(int);
     
-    inline arma_hot iterator& operator--();
-    inline arma_hot iterator  operator--(int);
+    inline arma_hot         iterator& operator--();
+    inline arma_warn_unused iterator  operator--(int);
     
     // this has a different value_type than iterator_base
     typedef SpValProxy<SpMat<eT> >         value_type;
@@ -443,11 +454,11 @@ class SpMat : public SpBase< eT, SpMat<eT> >
     inline const_row_iterator(const SpMat& in_M, uword in_row, uword in_col);
     inline const_row_iterator(const const_row_iterator& other);
     
-    inline arma_hot const_row_iterator& operator++();
-    inline arma_hot const_row_iterator  operator++(int);
+    inline arma_hot         const_row_iterator& operator++();
+    inline arma_warn_unused const_row_iterator  operator++(int);
     
-    inline arma_hot const_row_iterator& operator--();
-    inline arma_hot const_row_iterator  operator--(int);
+    inline arma_hot         const_row_iterator& operator--();
+    inline arma_warn_unused const_row_iterator  operator--(int);
     
     uword internal_row; // hold row internally because we use internal_pos differently
     uword actual_pos;   // actual position in matrix
@@ -482,11 +493,11 @@ class SpMat : public SpBase< eT, SpMat<eT> >
     inline arma_hot SpValProxy<SpMat<eT> > operator*();
     
     // overloads required for return type correctness
-    inline arma_hot row_iterator& operator++();
-    inline arma_hot row_iterator  operator++(int);
+    inline arma_hot         row_iterator& operator++();
+    inline arma_warn_unused row_iterator  operator++(int);
     
-    inline arma_hot row_iterator& operator--();
-    inline arma_hot row_iterator  operator--(int);
+    inline arma_hot         row_iterator& operator--();
+    inline arma_warn_unused row_iterator  operator--(int);
     
     // this has a different value_type than iterator_base
     typedef SpValProxy<SpMat<eT> >         value_type;
@@ -531,20 +542,24 @@ class SpMat : public SpBase< eT, SpMat<eT> >
   inline bool  empty() const;
   inline uword size()  const;
   
-  /**
-   * Resize memory.
-   * You are responsible for updating the column pointers and filling the new memory (if the new size is larger).
-   * If the new size is smaller, the first new_n_nonzero elements will be copied.
-   * n_nonzero is updated.
-   */
+  // Resize memory.
+  // If the new size is larger, the column pointers and new memory still need to be correctly set.
+  // If the new size is smaller, the first new_n_nonzero elements will be copied.
+  // n_nonzero is updated.
   inline void mem_resize(const uword new_n_nonzero);
   
+  //! synchronise CSC from cache
+  inline void sync() const;
+  
   //! don't use this unless you're writing internal Armadillo code
   inline void remove_zeros();
   
   //! don't use this unless you're writing internal Armadillo code
   inline void steal_mem(SpMat& X);
   
+  //! don't use this unless you're writing internal Armadillo code
+  inline void steal_mem_simple(SpMat& X);
+  
   //! don't use this unless you're writing internal Armadillo code
   template<              typename T1, typename Functor> arma_hot inline void init_xform   (const SpBase<eT, T1>& x, const Functor& func);
   template<typename eT2, typename T1, typename Functor> arma_hot inline void init_xform_mt(const SpBase<eT2,T1>& x, const Functor& func);
@@ -554,12 +569,15 @@ class SpMat : public SpBase< eT, SpMat<eT> >
   
   inline void init(uword in_rows, uword in_cols);
   inline void init(const std::string& text);
-  inline void init(const SpMat& x);
+  inline void init(const  SpMat<eT>& x);
+  inline void init(const MapMat<eT>& x);
   
   
   inline void init_batch_std(const Mat<uword>& locations, const Mat<eT>& values, const bool sort_locations);
   inline void init_batch_add(const Mat<uword>& locations, const Mat<eT>& values, const bool sort_locations);
   
+  inline SpMat(const arma_vec_indicator&, const uword in_vec_state);
+  inline SpMat(const arma_vec_indicator&, const uword in_n_rows, const uword in_n_cols, const uword in_vec_state);
   
   
   private:
@@ -574,14 +592,34 @@ class SpMat : public SpBase< eT, SpMat<eT> >
   arma_inline arma_hot arma_warn_unused uword get_position(const uword i) const;
   arma_inline arma_hot                  void  get_position(const uword i, uword& row_of_i, uword& col_of_i) const;
   
-  /**
-   * Add an element at the given position, and return a reference to it.
-   * The element will be set to 0 (unless otherwise specified).
-   * If the element already exists, its value will be overwritten.
-   */
-  inline arma_hot arma_warn_unused eT& add_element(const uword in_row, const uword in_col, const eT in_val = eT(0));
   
-  inline arma_hot void delete_element(const uword in_row, const uword in_col);
+  inline arma_warn_unused eT&  insert_element(const uword in_row, const uword in_col, const eT in_val = eT(0));
+  inline                  void delete_element(const uword in_row, const uword in_col);
+  
+  
+  // cache related
+  
+  arma_aligned mutable MapMat<eT> cache;
+  arma_aligned mutable state_type sync_state;
+  // 0: cache needs to be updated from CSC
+  // 1: CSC needs to be updated from cache
+  // 2: no update required
+  
+  #if !defined(ARMA_USE_OPENMP) && defined(ARMA_USE_CXX11)
+  arma_aligned mutable std::mutex cache_mutex;
+  #endif
+  
+  arma_inline void invalidate_cache() const;
+  arma_inline void invalidate_csc()   const;
+  
+  inline void sync_cache() const;
+  inline void sync_csc()   const;
+  
+  
+  friend class SpValProxy< SpMat<eT> >;  // allow SpValProxy to call insert_element() and delete_element()
+  friend class SpSubview<eT>;
+  friend class SpRow<eT>;
+  friend class SpCol<eT>;
   
   
   public:
