@@ -1,12 +1,17 @@
-// Copyright (C) 2012 National ICT Australia (NICTA)
+// Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
+// Copyright 2008-2016 National ICT Australia (NICTA)
 // 
-// This Source Code Form is subject to the terms of the Mozilla Public
-// License, v. 2.0. If a copy of the MPL was not distributed with this
-// file, You can obtain one at http://mozilla.org/MPL/2.0/.
-// -------------------------------------------------------------------
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+// http://www.apache.org/licenses/LICENSE-2.0
 // 
-// Written by Conrad Sanderson - http://conradsanderson.id.au
-// Written by Ryan Curtin
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+// ------------------------------------------------------------------------
 
 
 //! \addtogroup SpProxy
@@ -38,6 +43,7 @@ class SpProxy< SpMat<eT> >
     : Q(A)
     {
     arma_extra_debug_sigprint();
+    Q.sync();
     }
 
   arma_inline uword get_n_rows()    const { return Q.n_rows;    }
@@ -90,6 +96,7 @@ class SpProxy< SpCol<eT> >
     : Q(A)
     {
     arma_extra_debug_sigprint();
+    Q.sync();
     }
   
   arma_inline uword get_n_rows()    const { return Q.n_rows;    }
@@ -142,6 +149,7 @@ class SpProxy< SpRow<eT> >
     : Q(A)
     {
     arma_extra_debug_sigprint();
+    Q.sync();
     }
   
   arma_inline uword get_n_rows()    const { return 1;           }
@@ -194,6 +202,7 @@ class SpProxy< SpSubview<eT> >
     : Q(A)
     {
     arma_extra_debug_sigprint();
+    Q.m.sync();
     }
 
   arma_inline uword get_n_rows()    const { return Q.n_rows;    }
@@ -222,6 +231,58 @@ class SpProxy< SpSubview<eT> >
 
 
 
+template<typename eT>
+class SpProxy< spdiagview<eT> >
+  {
+  public:
+  
+  typedef eT                                       elem_type;
+  typedef typename get_pod_type<elem_type>::result pod_type;
+  typedef SpMat<eT>                                stored_type;
+  
+  typedef typename SpMat<eT>::const_iterator       const_iterator_type;
+  typedef typename SpMat<eT>::const_row_iterator   const_row_iterator_type;
+  
+  static const bool use_iterator   = false;
+  static const bool Q_is_generated = true;
+  
+  static const bool is_row = false;
+  static const bool is_col = true;
+  
+  arma_aligned const SpMat<eT> Q;
+  
+  inline explicit SpProxy(const spdiagview<eT>& A)
+    : Q(A)
+    {
+    arma_extra_debug_sigprint();
+    }
+  
+  arma_inline uword get_n_rows()    const { return Q.n_rows;    }
+  arma_inline uword get_n_cols()    const { return 1;           }
+  arma_inline uword get_n_elem()    const { return Q.n_elem;    }
+  arma_inline uword get_n_nonzero() const { return Q.n_nonzero; }
+  
+  arma_inline elem_type operator[](const uword i)                    const { return Q[i];           }
+  arma_inline elem_type at        (const uword row, const uword col) const { return Q.at(row, col); }
+  
+  arma_inline const eT*    get_values()      const { return Q.values;      }
+  arma_inline const uword* get_row_indices() const { return Q.row_indices; }
+  arma_inline const uword* get_col_ptrs()    const { return Q.col_ptrs;    }
+  
+  arma_inline const_iterator_type     begin()                            const { return Q.begin();            }
+  arma_inline const_iterator_type     begin_col(const uword col_num)     const { return Q.begin_col(col_num); }
+  arma_inline const_row_iterator_type begin_row(const uword row_num = 0) const { return Q.begin_row(row_num); }
+  
+  arma_inline const_iterator_type     end()                        const { return Q.end();            }
+  arma_inline const_row_iterator_type end_row()                    const { return Q.end_row();        }
+  arma_inline const_row_iterator_type end_row(const uword row_num) const { return Q.end_row(row_num); }
+  
+  template<typename eT2>
+  arma_inline bool is_alias(const SpMat<eT2>&) const { return false; }
+  };
+
+
+
 template<typename T1, typename spop_type>
 class SpProxy< SpOp<T1, spop_type> >
   {
@@ -334,7 +395,6 @@ class SpProxy< mtSpOp<out_eT, T1, spop_type> >
   public:
   
   typedef          out_eT                          elem_type;
-  typedef typename T1::elem_type                   eT;
   typedef typename get_pod_type<elem_type>::result pod_type;
   typedef SpMat<out_eT>                            stored_type;
   
@@ -363,9 +423,9 @@ class SpProxy< mtSpOp<out_eT, T1, spop_type> >
   arma_inline elem_type operator[](const uword i)                    const { return Q[i];           }
   arma_inline elem_type at        (const uword row, const uword col) const { return Q.at(row, col); }
   
-  arma_inline const eT*    get_values()      const { return Q.values;      }
-  arma_inline const uword* get_row_indices() const { return Q.row_indices; }
-  arma_inline const uword* get_col_ptrs()    const { return Q.col_ptrs;    }
+  arma_inline const out_eT* get_values()      const { return Q.values;      }
+  arma_inline const uword*  get_row_indices() const { return Q.row_indices; }
+  arma_inline const uword*  get_col_ptrs()    const { return Q.col_ptrs;    }
   
   arma_inline const_iterator_type     begin()                            const { return Q.begin();            }
   arma_inline const_iterator_type     begin_col(const uword col_num)     const { return Q.begin_col(col_num); }
